DATA_SECTION
  init_int noObs
  init_matrix obs(1,noObs,1,2)
  vector X0(1,2)  
  
PARAMETER_SECTION
  init_vector logK(1,3);
  init_number logSigma;
  init_number logTau(2);
  init_number logRho(2);

  sdreport_vector k(1,3);
  sdreport_number sigma2;
  sdreport_number tau2;
  sdreport_number rho2;
  sdreport_vector M(1,noObs);

  matrix X(1,noObs,1,2);
  matrix A(1,2,1,2);
  matrix S(1,noObs,1,noObs);
  sdreport_vector Mres(1,noObs);
  objective_function_value nll;

PRELIMINARY_CALCS_SECTION
  X0(1)=0.0; X0(2)=100.0;
  logK=-2.0;
  logSigma=-2;
  logRho=2;
  logTau=-1;

PROCEDURE_SECTION
  k=exp(logK);
  sigma2=exp(2.0*logSigma);
  tau2=exp(2.0*logTau);
  rho2=exp(2.0*logRho);

  A(1,1)= -k(1); A(1,2)=  k(2);      
  A(2,1)=  k(1); A(2,2)= -k(2)-k(3); 

  S.initialize();
  for(int i=1; i<=noObs; ++i){
    for(int j=i; j<=noObs; ++j){
      S(i,j)=tau2*exp(-square(obs(i,1)-obs(j,1))/rho2);
      S(j,i)=S(i,j);
    }
    S(i,i)+=sigma2;
  }

  for(int i=1; i<=noObs; ++i){
    X(i)=expm(A*obs(i,1))*X0;
    M(i)=100.0-sum(X(i));
    Mres(i)=obs(i,2)-M(i);
  }

  nll=0.5*(log(2.0*M_PI)*noObs+log(det(S))+Mres*inv(S)*Mres);

REPORT_SECTION
  for(int i=1; i<=noObs; ++i){
    report<<obs(i,1)<<" "<<obs(i,2)<<" "<<X(i,1)<<" "<<X(i,2)<<" "<<M(i)<<endl;
  }