# Data

library(nlme)
ortho <- with(Orthodont[Orthodont$Sex=="Female",], data.frame(Subject=factor(as.character(Subject)), age=age, distance=distance))



# Simple LM
ortho.simple <- lm(distance~age, data=ortho)
coef(ortho.simple)
summary(ortho.simple)$sigma

orthofit <- rbind(ortho, ortho)
orthofit$Series <- rep(c("Obs","Pred"), each=44)
orthofit$distance[orthofit$Series=="Pred"] <- fitted(ortho.simple)
xyplot(distance~age|Subject, groups=Series, data=orthofit, layout=c(11,1), panel=panel.superpose.2, type=c("p","l"), col=1, lwd=2)



# LM

ortho.lm <- lm(distance~Subject+age, data=ortho)
coef(ortho.lm)
summary(ortho.lm)$sigma

orthofit <- rbind(ortho, ortho)
orthofit$Series <- rep(c("Obs","Pred"), each=44)
orthofit$distance[orthofit$Series=="Pred"] <- fitted(ortho.lm)
xyplot(distance~age|Subject, groups=Series, data=orthofit, layout=c(11,1), panel=panel.superpose.2, type=c("p","l"), col=1, lwd=2)



# LME

ortho.lme <- lme(distance~age, random=~1|Subject, data=ortho)
fixef(ortho.lme)
print(summary(ortho.lme$modelStruct), ortho.lme$sigma)
ranef(ortho.lme)

ortho.lme2 <- update(ortho.lme, method="ML")
fixef(ortho.lme2)
print(summary(ortho.lme2$modelStruct), ortho.lme2$sigma)
ranef(ortho.lme2)

orthofit <- rbind(ortho, ortho, ortho, ortho)
orthofit$Series <- rep(c("data","lm","lme","lme2"), each=44)
orthofit$distance <- c(ortho$distance, fitted(ortho.lm), fitted(ortho.lme), fitted(ortho.lme2))
xyplot(distance~age|Subject, groups=Series, data=orthofit, layout=c(11,1), panel=panel.superpose.2, type=c("p","l","l","l"), col=c(1,1,2,4), lwd=2)



# ADMB-RE

data.frame(unclass(xtabs(distance~age+Subject, data=ortho)))
